#!/usr/bin/perl
#####################################################################
# Function: common.pm
#
#
# 概要:
# PG-REX 便利ツールから呼び出すモジュールの集まり
# 
# 特記事項:
# なし
#
#####################################################################
use warnings;
use strict;
use Switch;
use Net::OpenSSH;
use include::command;

BEGIN {
    if ($ENV{"LANG"} =~ m/ja/i){
        eval qq{
            use po::ja;
        };
    }
    else{
        eval qq{
            use po::en;
        };
    }
};


sub postgres_running{
    my ($pgdata) = @_;
    my $result;

    $result = exec_command("$PS aux | $GREP \"bin/postgres -D $pgdata \" | $GREP -v \"grep\"");

    # PostgreSQL のプロセスを発見したら1を返却する
    if ($result ne ""){
        return 1;
    }
    return 0;
}


sub pacemaker_running{
    my $result;

    $result = exec_command("$PS aux | $GREP \"heartbeat: master control process\" | $GREP -v \"grep\"");

    # Pacemaker のプロセスを発見したら1を返却する
    if ($result ne ""){
        return 1;
    }
    return 0;
}


sub pacemaker_online{
    my ($my_node) = @_;
    my @results;
    my @crm_results;
    my $exit_code;

    $results[0] = `$CRM_MON -fA -1`;
    $results[1] = $? >> 8;

    $exit_code = $results[1];
    # Pacemaker が起動している場合は $exit_code = 0 、起動していない場合は $exit_code = 10
    if ($exit_code != 0 && $exit_code != 10){
        printlog("ERROR", COMMON_MS0001);
    }

    # crm_mon の結果を行ごとの配列に格納する
    @crm_results = split (/\n/, $results[0]);

    # Pacemaker が Online だったら1を返却する
    foreach my $line (@crm_results){
        # crm_mon の結果に「Online: [ ～ <マシンのホスト名>」の行が存在すると OK
        if ($line =~ /Online\:\s+\[(\s+\S+)?\s+$my_node\s+/){
            return 1;
        }
    }
    return 0;

}


sub pgrex_failed_action{
    my ($my_node) = @_;
    my @results;
    my @crm_results;
    my $exit_code;

    $results[0] = `$CRM_MON -fA -1`;
    $results[1] = $? >> 8;

    $exit_code = $results[1];
    # Pacemaker が起動している場合は $exit_code = 0 、起動していない場合は $exit_code = 10
    if ($exit_code != 0 && $exit_code != 10){
        printlog("ERROR", COMMON_MS0001);
    }
    
    # crm_mon の結果を行ごとの配列に格納する
    @crm_results = split (/\n/, $results[0]);

    my $line_count = 0;
    my $line_tmp = 0;
    # crm_mon の結果に failed Action の内容が存在していたら1を返却する
    foreach my $line (@crm_results){
        $line_count++;

        # crm_mon の結果に「Failed actions:」の行が存在した後、
        # 1行後に「pgsql:(.) ～ node=<マシンのホスト名>」の行が存在すると OK
        # ※(.) = 文字の種類は問わず一文字
        if ($line =~ /Failed\s+actions:/){
            $line_tmp = $line_count;
        }
        if ($line =~ /pgsql:[0-1]+_\S+_[0-9]+\s+\(node=$my_node/ && $line_count == $line_tmp + 1){
            return 1;
        }
    }
    return 0;

}


sub network_running{
    my ($my_node, $another_node, @nic_list) = @_;
    my $result;
    my $attribute;
    my $exit_code;

    # コマンド
    # 「crm_attribute -t status -N <ノード名> --name <ノード名>-<NIC名> -G -q」
    # の結果が2ノード×2nicの計4つとも"up"であると OK
    foreach my $nic (@nic_list){
        $attribute = $another_node."-".$nic;
        $result = `$CRM_ATTRIBUTE -t status -N $my_node --name $attribute -G -q 2> /dev/null`;
        chomp $result;
        if ($result ne "up"){
            return 0;
        }
    }
    foreach my $nic (@nic_list){
        $attribute = $my_node."-".$nic;
        $result = `$CRM_ATTRIBUTE -t status -N $another_node --name $attribute -G -q 2> /dev/null`;
        chomp $result;
        if ($result ne "up"){
            return 0;
        }
    }
    return 1;

}


sub master_running{
    my ($my_node, $master_resource_id, $primitive_resource_id, $ssh_pass, $address_connect) = @_;
    my @results;
    my $command;

    ##
    # 以下の条件を満たしたとき結果を OK とする
    #   *  コマンド
    #     「crm_resource -r <MasterリソースID> -W 2> /dev/null | grep " <マシンのホスト名> "」
    #      の結果が"resource msPostgresql is running on: <マシンのホスト名> Master"である
    #   *  コマンド
    #     「crm_attribute -t status -N <マシンのホスト名> --name master-<PrimitiveリソースID>:0 -G -q」
    #     「crm_attribute -t status -N <マシンのホスト名> --name master-<PrimitiveリソースID>:1 -G -q」
    #      のどちらも実行して、どちらか結果が"1000"である
    #   *  コマンド
    #     「crm_attribute -l forever -N <マシンのホスト名> --name pgsql-data-status -G -q」
    #      の結果が"LATEST"である
    #   *  コマンド
    #     「crm_attribute -t status -N <マシンのホスト名> --name pgsql-status -G -q」
    #      の結果が"PRI"である
    ##

    $command = "$CRM_RESOURCE -r $master_resource_id -W 2> /dev/null | $GREP \" $my_node \"";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] ne "resource $master_resource_id is running on: $my_node Master"){
        return 0;
    }

    $command = "$CRM_ATTRIBUTE -l forever -N $my_node --name pgsql-data-status -G -q 2> /dev/null";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] ne "LATEST"){
        return 0;
    }

    $command = "$CRM_ATTRIBUTE -t status -N $my_node --name pgsql-status -G -q 2> /dev/null";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] ne "PRI"){
        return 0;
    }

    $command = "$CRM_ATTRIBUTE -t status -N $my_node --name master-$primitive_resource_id:0 -G -q 2> /dev/null";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] eq "1000"){
        return 1;
    }

    $command = "$CRM_ATTRIBUTE -t status -N $my_node --name master-$primitive_resource_id:1 -G -q 2> /dev/null";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] eq "1000"){
        return 1;
    }

    return 0;
}


sub slave_running{
    my ($my_node, $master_resource_id, $primitive_resource_id, $ssh_pass, $address_connect) = @_;
    my @results;
    my $command;

    ##
    # 以下の条件を満たしたとき結果を OK とする
    #   *  コマンド
    #     「crm_resource -r <MasterリソースID> -W 2> /dev/null | grep " <マシンのホスト名> "」
    #      の結果が"resource msPostgresql is running on: <マシンのホスト名> "である
    #   *  コマンド
    #     「crm_attribute -t status -N <マシンのホスト名> --name master-<PrimitiveリソースID>:1 -G -q」
    #     「crm_attribute -t status -N <マシンのホスト名> --name master-<PrimitiveリソースID>:0 -G -q」
    #      のどちらも実行して、どちらかの結果が"100"である
    #   *  コマンド
    #     「crm_attribute -l forever -N <マシンのホスト名> --name pgsql-data-status -G -q」
    #      の結果が"STREAMING|SYNC"である
    #   *  コマンド
    #     「crm_attribute -t status -N <マシンのホスト名> --name pgsql-status -G -q」
    #      の結果が"HS:SYNC"である
    ##

    $command = "$CRM_RESOURCE -r $master_resource_id -W 2> /dev/null | $GREP \" $my_node \"";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] ne "resource $master_resource_id is running on: $my_node "){
        return 0;
    }

    $command = "$CRM_ATTRIBUTE -l forever -N $my_node --name pgsql-data-status -G -q 2> /dev/null";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] ne "STREAMING|SYNC"){
        return 0;
    }

    $command = "$CRM_ATTRIBUTE -t status -N $my_node --name pgsql-status -G -q 2> /dev/null";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] ne "HS:sync"){
        return 0;
    }

    $command = "$CRM_ATTRIBUTE -t status -N $my_node --name master-$primitive_resource_id:1 -G -q 2> /dev/null";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] eq "100"){
        return 1;
    }

    $command = "$CRM_ATTRIBUTE -t status -N $my_node --name master-$primitive_resource_id:0 -G -q 2> /dev/null";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] eq "100"){
        return 1;
    }

    return 0;

}


sub vip_running{
    my ($my_node, $resource_id) = @_;
    my $result;

    ##
    # 以下の条件を満たしたとき結果を OK とする
    #   *  コマンド
    #     「crm_resource -r <リソースID> -W 2> /dev/null」
    #      の結果が"resource <リソースID> is running on: <マシンのホスト名> "である
    ##

    $result = `$CRM_RESOURCE -r $resource_id -W 2> /dev/null`;
    chomp $result;
    if ($result ne "resource $resource_id is running on: $my_node "){
        return 0;
    }

    return 1;

}


sub stonith_running{
    my ($my_node, $resource_id) = @_;
    my $result;

    ##
    # 以下の条件を満たしたとき結果を OK とする
    #   *  コマンド
    #     「crm_resource -r <リソースID_1> -W 2> /dev/null」
    #     「crm_resource -r <リソースID_2> -W 2> /dev/null」
    #      のどちらも実行して、どちらかの結果が
    #      "resource <指定したリソースID> is running on: <マシンのホスト名> "である
    ##

    $result = `$CRM_RESOURCE -r $resource_id->[0] -W 2> /dev/null`;
    chomp $result;
    if ($result eq "resource $resource_id->[0] is running on: $my_node "){
        return 1;
    }

    $result = `$CRM_RESOURCE -r $resource_id->[1] -W 2> /dev/null`;
    chomp $result;
    if ($result eq "resource $resource_id->[1] is running on: $my_node "){
        return 1;
    }

    return 0;

}


sub pingd_running{
    my ($my_node, $resource_id, $attribute_name) = @_;
    my $result;

    ##
    # 以下の条件を満たしたとき結果を OK とする
    #   *  コマンド
    #     「crm_resource -r <リソースID> -W 2> /dev/null | grep " <マシンのホスト名> "」
    #      の結果が"resource <リソースID> is running on: <マシンのホスト名> "である
    #   *  コマンド
    #     「crm_attribute -t status -N <マシンのホスト名> --name <属性名> -G -q」
    #      の結果が"100"である
    ##

    $result = `$CRM_RESOURCE -r $resource_id -W 2> /dev/null | $GREP \" $my_node \"`;
    chomp $result;
    if ($result ne "resource $resource_id is running on: $my_node "){
        return 0;
    }

    $result = `$CRM_ATTRIBUTE -t status -N $my_node --name $attribute_name -G -q 2> /dev/null`;
    chomp $result;
    if ($result ne "100"){
        return 0;
    }

    return 1;

}


sub diskd_running{
    my ($my_node, $resource_id, $attribute_name) = @_;
    my $result;

    ##
    # 以下の条件を満たしたとき結果を OK とする
    #   *  コマンド
    #     「crm_resource -r <リソースID> -W 2> /dev/null | grep " <マシンのホスト名> "」
    #      の結果が"resource <リソースID> is running on: <マシンのホスト名> "である
    #   *  コマンド
    #     「crm_attribute -t status -N <マシンのホスト名> --name <属性名> -G -q」
    #      の結果が"normal"である
    ##

    $result = `$CRM_RESOURCE -r $resource_id -W 2> /dev/null | $GREP \" $my_node \"`;
    chomp $result;
    if ($result ne "resource $resource_id is running on: $my_node "){
        return 0;
    }

    $result = `$CRM_ATTRIBUTE -t status -N $my_node --name $attribute_name -G -q 2> /dev/null`;
    chomp $result;
    if ($result ne "normal"){
        return 0;
    }

    return 1;

}


sub read_cib{
    my ($cib_path, $pg_basebackup_path, $pg_primitive_resource_id, $kill_when_no_data) = @_;
    my @cib_strings;
    my %cib_value;
    my $my_node;
    my $my_node_id;
    my $my_pgsql_data_status_id;
    my @check_keylist = ("pgdata", "master_ip", "repuser");
    my $result;

    $my_node = exec_command("$UNAME -n");
    chomp $my_node;

    # PG-REX のバージョンを確認し、その結果に合わせて tmpdir のデフォルト値を設定する
    $result = get_pg_version($pg_basebackup_path);
    if ($result =~ /^9\.1[^0-9]/){
        $cib_value{"tmpdir"} = "/var/lib/pgsql";
    }
    else{
        $cib_value{"tmpdir"} = "/var/lib/pgsql/tmp";
    }

    open (FILE, $cib_path) or printlog("ERROR", COMMON_MS0023, $cib_path);
    @cib_strings = <FILE>;
    close (FILE);

    # PostgreSQL のリソース部分からパラメータ値を取得
    foreach my $line (@cib_strings){
        # <PostgreSQLのPrimitiveのリソースID>-instance_attributes format : 
        #     <nvpair id="<PostgreSQLのPrimitiveのリソースID>-instance_attributes-<文字列>" name="<name>" value="<value>"/>
        if ($line =~ /\"$pg_primitive_resource_id-instance_attributes-\S+\"\s+name=\"([^\"\s]+)\"\s+value=\"([^\"]+)\"/){
            $cib_value{$1} = $2;
        }
        # node id format : 
        #     <node (id="<node id>" type="<type>" uname="<node name>")>
        #     ※()の中のタグの順番は不定
        if ($line =~ /uname=\"$my_node\"/ && $line =~ /id=\"([^\"\s]+)\"/){
            $my_node_id = $1;
            $my_pgsql_data_status_id = "nodes-".$my_node_id."-pgsql-data-status";
        }
    }

    foreach my $line (@cib_strings){
        if ($my_pgsql_data_status_id && $line =~ /id=\"$my_pgsql_data_status_id\"/ && $line =~ /value=\"([^\"\s]+)\"/){
            $cib_value{"pgsql_data_status"} = $1;
        }
    }
    
    foreach my $key (@check_keylist){
        if (!exists($cib_value{$key}) && !$kill_when_no_data){
            printlog("ERROR", COMMON_MS0003, $pg_primitive_resource_id, $key);
        }
    }

    return %cib_value;
}


sub read_config{
    my ($config_path) = @_;
    my @config_strings;
    my %config_value;
    my @check_keylist = ("Archive_dir", "D-LAN_IPAddress", 
                         "PG-REX_Master_ResourceID","PG-REX_Primitive_ResourceID");
    my $result;

    $config_value{'STONITH'} = "enable";
    $config_value{'VIP_SLAVE'} = "enable";

    open (FILE, $config_path) or printlog("ERROR", COMMON_MS0024, $config_path);
    @config_strings = <FILE>;
    close (FILE);

    foreach my $line (@config_strings){
        # コメントの削除
        # comment format : #<comment>
        $line =~ s/\#.*//g;

        # 行末の空白の削除
        # space format : <space><\n>
        $line =~ s/\s+$//g;

        # 全角の空白を半角空白に変換
        # two-byte characterspace format : <two-byte character space>
        $line =~ s/　/ /g;

        # environment setting value format : <<value name> = <parameter>> or <<value name> = <parameter>,<parameter>>
        if ($line =~ /^\s*(\S+)\s*=\s*(.+)$/){
            $config_value{$1} = $2;
        }
    }

    foreach my $key (@check_keylist) {
        if (!exists($config_value{$key})){
            printlog("ERROR", COMMON_MS0004, $key);
        }
    }
    if ($config_value{'Archive_dir'} !~ /^\//){
        printlog("ERROR", COMMON_MS0005);
    }
    if ($config_value{'STONITH'} ne "enable" && $config_value{'STONITH'} ne "disable"){
        printlog("ERROR", COMMON_MS0006);
    }
    if ($config_value{'VIP_SLAVE'} ne "enable" && $config_value{'VIP_SLAVE'} ne "disable"){
        printlog("ERROR", COMMON_MS0021);
    }

    # VIP_SLAVE が disable の場合、VIP-SLAVE_ResourceID の値を空にする
    if ($config_value{'VIP_SLAVE'} eq "disable"){
        $config_value{"VIP-SLAVE_ResourceID"} = "";
    }

    # STONITH が enable の場合、 Group のリソース ID を配列に分解しハッシュに格納する
    # また、disable の場合、STONITH_ResourceID の値を空にする
    if ($config_value{'STONITH'} eq "enable" && exists($config_value{"STONITH_ResourceID"})){
        my @stonith_resources = split(/\s*,\s*/, $config_value{"STONITH_ResourceID"});
        if (scalar(@stonith_resources) != 2){
            printlog("ERROR", COMMON_MS0022);
        }
        $config_value{"STONITH_ResourceID"} = [$stonith_resources[0], $stonith_resources[1]];
    }
    else{
        $config_value{"STONITH_ResourceID"} = "";
    }

    my @ifconfig_ip = ();
    # ","で区切られた D-LAN の IP アドレスをそれぞれ配列に格納
    my @d_lan_ip = split(/\s*,\s*/, $config_value{"D-LAN_IPAddress"});
    if (scalar(@d_lan_ip) != 2){
        printlog("ERROR", COMMON_MS0007);
    }

    # コマンド実行マシンに設定してある IP アドレスを配列に格納
    $result = exec_command("$IFCONFIG | $GREP \"inet addr\"");
    my @ifconfig_strings = split(/\n/, $result);
    foreach my $line (@ifconfig_strings){
        # IP address format : <inet addr:<value>>
        if ($line =~ /inet\saddr:([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)/) {
            push (@ifconfig_ip , $1);
        }
    }
    if (!scalar(@ifconfig_ip)){
        printlog("ERROR", COMMON_MS0008);
    }

    foreach my $line (@d_lan_ip){
        if (grep {$_ eq $line} @ifconfig_ip){
            $config_value{"My_D-LAN_IPAddress"} = $line;
        }
        else {
            $config_value{"Another_D-LAN_IPAddress"} = $line;
        }
    }

    if (!exists($config_value{'My_D-LAN_IPAddress'})){
        printlog("ERROR", COMMON_MS0009);
    }
    if (!exists($config_value{'Another_D-LAN_IPAddress'})){
        printlog("ERROR", COMMON_MS0010);
    }

    return %config_value;
}


sub read_hacf{
    my ($hacf_path) = @_;
    my @hacf_strings;
    my %hacf_value;
    my @ic_dev;
    my @node;

    open (FILE, $hacf_path) or printlog("ERROR", COMMON_MS0025, $hacf_path);
    @hacf_strings = <FILE>;
    close (FILE);

    # IC-LAN に使用しているデバイス名とノード名を取得
    foreach my $line (@hacf_strings){
        chomp $line;
        # comment format : #<comment>
        $line =~ s/\#.*//g;
        # IC-LAN device name format : bcast <device name> or ucast <device name>
        if ($line =~ /^\s*(b|u)cast\s+(\S+)/){
            push(@ic_dev, $2);
        }
        # node name format : node <node name>
        if ($line =~ /^\s*node\s+(\S+)/){
            push(@node, $1);
        }
    }
    if (scalar(@ic_dev) != 2){
        printlog("ERROR", COMMON_MS0019);
    }
    if (scalar(@node) != 2){
        printlog("ERROR", COMMON_MS0020);
    }

    %hacf_value = (ic_dev => [$ic_dev[0], $ic_dev[1]],
                   node   => [$node[0], $node[1]]);

    return %hacf_value;
}


sub exec_command {
    my ($command) = @_;

    my $return = `$command`;
    my $exit_code = $? >> 8;
    if ($exit_code != 0 && $exit_code != 1){
        printlog("ERROR", COMMON_MS0011, $command);
    }

    return $return;
}


sub ssh_exec_command {
    my ($address, $user, $pass, $command) = @_;
    my @results;
    
    my $ssh = Net::OpenSSH->new($address, user => $user, password => $pass);
    $ssh->error and printlog("ERROR", COMMON_MS0016);
    
    $results[0] = $ssh->capture($command);
    $results[1] = $? >> 8;
    chomp $results[0];
    
    return @results;
}


sub rsync_exec_command {
    my ($address, $user, $pass, $remote, $local) = @_; 
    
    my $ssh = Net::OpenSSH->new($address, user => $user, password => $pass);
    $ssh->error and printlog("ERROR", COMMON_MS0016);
    
    my %opts = (
        archive => 1,
        verbose => 1,
    );
    
    $ssh-> rsync_get(\%opts, $remote, $local) or $ssh->error and printlog("ERROR", COMMON_MS0017);
    
    return 0;
}


sub scp_exec_command {
    my ($address, $user, $pass, $remote, $local) = @_; 
    
    my $ssh = Net::OpenSSH->new($address, user => $user, password => $pass);
    $ssh->error and printlog("ERROR", COMMON_MS0016);
    
    $ssh-> scp_get({copy_attrs => 1}, $remote, $local) or $ssh->error and printlog("ERROR", COMMON_MS0018);
    
    return 0;
}


sub get_xlogfile_name{
    my ($location, $ThisTimeLineID) = @_;
    my $uxlogid;
    my $uxrecoff;
    my $xlogid;
    my $xlogseg;
    my $xlogfilename;
    
    # XLOG 位置の情報をパース
    # location format : <xlogid>/<xrecoff>
    # <xlogid>は1,2桁の16進数
    # <xrecoff>は7.8桁の16進数
    if ($location !~ /^([0-9A-F]+)\/([0-9A-F]{7,8})$/){
        printlog("ERROR", COMMON_MS0013);
    }
    $uxlogid = $1;
    $uxrecoff = $2;

    # 一つ前のセグメントを取得
    # $xlogsegは、上で取得した16進数の値から1を引いて5桁以下をシフトした値になる
    $xlogid = hex($uxlogid);
    my $uxrecoff_tmp = hex($uxrecoff);
    $xlogseg = sprintf("%d", ($uxrecoff_tmp - 1)/(16 ** 6));

    # 一つ前のセグメントの WAL ファイル名を取得
    $xlogfilename = sprintf("%08X%08X%08X", $ThisTimeLineID, $xlogid, $xlogseg);

    return $xlogfilename;
}


sub get_controldata_value{
    my (@controldata_strings) = @_;
    my %controldata_value;
    my @key_list = ("updatetime", "database", "timelineid", "xloglocation");

    foreach my $line (@controldata_strings){
        # pg_control last modified format : pg_control last modified: <Wday> <Mon> <Mday> <Hour>:<Min>:<Sec> <Year>
        if ($line =~ /^pg_control\s+last\s+modified:\s+(.+)$/){
            $controldata_value{'updatetime'} = $1;
        }
        # Database system identifier format : Database system identifier: <Database>
        if ($line =~ /^Database\s+system\s+identifier:\s+([0-9A-F]+)$/){
            $controldata_value{'database'} = $1;
        }
        # TimeLineID format : Latest checkpoint's TimeLineID: <TimeLineID>
        # <TimeLineID>は10進数
        if ($line =~ /^Latest\s+checkpoint's\s+TimeLineID:\s+([0-9A-F]+)$/){
            $controldata_value{'timelineid'} = $1;
        }
        # location format : Latest checkpoint's REDO location: <location>
        if ($line =~ /^Latest\s+checkpoint's\s+REDO\s+location:\s+([0-9A-F\/]+)$/){
            $controldata_value{'xloglocation'} = $1;
        }
    }

    foreach my $key (@key_list) {
        if (!exists($controldata_value{$key})){
            printlog("ERROR", COMMON_MS0014);
        }
    }


    return %controldata_value;
}


sub get_recoverywal {
    my (@controldata_strings) = @_;
    my $wal = '';

    my %controldata_value = get_controldata_value(@controldata_strings);

    $wal = get_xlogfile_name($controldata_value{'xloglocation'}, $controldata_value{'timelineid'});

    return $wal;
}


sub get_node{
    my ($another_address,$ssh_pass) = @_;
    my @results;
    my %node_value = ("my_node"      => '',
                      "another_node" => '');

    $node_value{'my_node'} = exec_command("$UNAME -n");
    chomp $node_value{'my_node'};

    @results = ssh_exec_command($another_address, "root", $ssh_pass, "$UNAME -n");
    $node_value{'another_node'} = $results[0];
    chomp $node_value{'another_node'};

    if (!$node_value{'another_node'}){
        printlog("ERROR", COMMON_MS0015);
    }

    return %node_value;
}


sub get_pg_version{
    my ($pg_basebackup_path) = @_;
    my $result;
    my @results;

    $result = exec_command("$SU - postgres -c \"$pg_basebackup_path --version \"");

    @results = split(/\s/,$result);
    chomp $results[2];

    return $results[2];
}

sub get_pg_command_path {
    my ($path) = @_;
    my %commandpath;
    my @command_list = ("psql", "pg_controldata", "pg_basebackup");

    if (!$path){
        $path = "/usr/pgsql-9.2/bin";
    }
    else{
        $path =~ s/\/$//;
    }

    foreach my $command (@command_list){
        $commandpath{$command} = `$SU - postgres -c "$WHICH $command" 2> /dev/null`;
        if (!$commandpath{$command}){
            $commandpath{$command} = $path."/".$command;
        }
        else{
            chomp $commandpath{$command};
        }
    }

    return %commandpath;
}

sub check_user{
    my $result;

    $result = exec_command("$WHOAMI");
    chomp $result;
    if ($result ne "root"){
        printlog("ERROR", COMMON_MS0026, $result);
    }
}

sub printlog {
    my ($log_level, $message, @values) = @_;

    for (my $count=0; $count < scalar(@values); $count++){
        $message =~ s/\[$count\]/$values[$count]/;
    }

    if ($log_level eq "ERROR"){
        die $message;
    }
    else{
        print $message;
    }

}


1;
